;=====================================================================================
; Ivan Smarzhevskiy
; Personnel decisions in the hierarchy
;
; Peoples Friendship University of Russia, Moscow, Russian Federation
;
; Copyright 2022 Ivan Smarzhevskiy
;
;
; Corresponding author contact:  Ivan Smarzhevskiy  - ivsmrudn@yandex.ru  IvanSmarzhevskiy@gmail.com>
;
;=====================================================================================
globals
[
 UnitsQty xx yy v u vp vpx vpy unitValue HumanList LastChangeTick ChangesQty PowerNoCoalitionAgents HarismaMean
 PF0 PF1 PF2 PB0 PB1 PB2 PC0 PC1 PC2 CF0 CF1 CF2 CB0 CB1 CB2 CC0 CC1 CC2 VDm DmAll P2Frnd P2Frnd3 DmWorst P2NBU DwnGr DwnGrDism UpGr GetSub GetFreSub
 GetAnySub ReplSub
 PotentialProductivity ActualProductivityHistory MissedProductivityHistory MeanActualProductivityHistory MeanMissedProductivityHistory
 CoalitionsQty0
]
;; service variables are  xx yy v u vp vpx vpy


breed [units unit]
breed [humans human]
breed [coalitions coalition]
;;------------------------------------------------------

units-own
[
  level          ;; in hierarhy
  my_boss        ;; who of my boss OR -1 (-1 only for top position)

  Value          ;; value of the unit
  personID       ;; ID of person placed at unit (who of person) OR -1
  history_list   ;; list for person's IDs, assigned to unit throw the simulation run
  Busytime       ;; time (sum of ticks) during the unit was occupied
  D_list         ;; list of  Decisions, making by Humans on the positon (unit) throw the simulation run (calling by "D_procedure who")
  originalcolor  ;; color at birth

  OAIndex        ;; Operational Activity Index
  HSum           ;; sum of suborder's harismas
  PQty           ;; quantity of suborders

  PercptSumOfCurrHuman   ;;
]

humans-own
[
  harisma             ;; the measure of personal productivity
  coalitionID         ;; ID of coalition OR -1
  MyObjectives_list   ;; list of sites to occupy
  PerceptOfOther      ;; list (with two possible values values -1 and 1) of my perception of other humans (in order of their who)

  site                ;; ID of site placed (who of Unit assigned) OR -1 for free humans
  history             ;; list for unit's who, human has assigned to throw the simulation run
  onunitstime         ;; total time period (sum of ticks) of units occupation
  onL1time            ;; total time period on Level 1 units
  onL2time            ;; total time period on Level 2 units
  onL3time            ;; total time period on Level 3 units
  freetime            ;; total time period no units occupation
  ValuePerCareer      ;; sum of value throw on-units placing
  ObjectOfDec         ;; list of decisions, where human was the object of
]

coalitions-own
[
  SumOfLevels      ;; sum of unit's ocuppied by coalition's members levels
  power            ;; total power of coalition (see CoalitionsPower plot)


  Objectives_list  ;; list of sites to occupy
]
;;   links-own [birthdate]
;;---------------------------------------------------------------------SETUP---------------------------------------
to setup
  clear-all
  ask patches   [ set pcolor 67 ]    ;;;; For 3d View

  SetHumans
  SetUnits

  ;; There is variants of first assignments. You can combine some downlined procedures, for example as it implemented in code now.

  AssignTop                  ;; In first we assign human on unit with level 0
  ;; AssignHeads             ;; In first we assign humans to all (no matter of FirstDistrDensity) Head's Units... ;;
  ;; AssignSuborders         ;; ... and then - to units with level = 3                                            ;;
  AssignAnyUnits          ;; Assign humans to all level Units in according with FirstDistrDensity
  ;; AssignOnlySubs          ;; Assign humans only to level 3 Units (no matter for FirstDistrDensity)             ;;

  set CoalitionsQty0 CoalitionsQty
  SetCoalitions
  AssignPerceptions
  SetCoalitionPerceptions

;;-------------------------- Decision's statistic data -------------------------------------------------
  set ChangesQty 0
  set PF0 0 set PF1 0 set PF2 0 set PB0 0 set PB1 0 set PB2 0 set PC0 0 set PC1 0 set PC2 0
  set CF0 0 set CF1 0 set CF2 0 set CB0 0 set CB1 0 set CB2 0 set CC0 0 set CC1 0 set CC2 0
  set VDm 0 set DmAll 0 set P2Frnd 0 set DmWorst 0 set P2NBU 0 set P2Frnd3 0 set DwnGr 0
  set DwnGrDism 0 set UpGr 0 set GetSub 0 set GetFreSub 0 set GetAnySub 0 set ReplSub 0
  set PotentialProductivity sum [harisma] of max-n-of (min (list UnitsQty HumansQty)) humans [harisma]
  set ActualProductivityHistory []
  set MissedProductivityHistory []
  set MeanActualProductivityHistory []
  set MeanMissedProductivityHistory []
  ;;-------------------------------------------------------------------------------------------------------
  show "setup done!"
  reset-ticks
end
;;----------GO --------GO --------GO --------GO --------GO --------GO --------GO --------GO --------GO -----...
to go

 TotalPowerCalc
 SetPersonalObjectives
 PerceptSum4UnitCalc

  ;; hierarchy behaviour Screeplay 1,2 ...
  (ifelse ScreenPlay = "SP1 (No Lev2 Ctrl)"    [sp1]
          ScreenPlay = "SP2 (3 Points Ctrl)"   [sp2]
          ScreenPlay = "SP3 (No Lev 0 Ctrl)"   [sp3]
          ScreenPlay = "SP4 (Random)"          [sp4]
          ScreenPlay = "SP5 No Control"        [sp5]
  )

  ask humans with [site != -1]
  [
    if [level] of unit site = 1 [set onL1time onL1time  + 1]
    if [level] of unit site = 2 [set onL2time onL2time  + 1]
    if [level] of unit site = 3 [set onL3time onL3time  + 1]
    set OnUnitstime OnUnitstime + 1
          ifelse OnUnitstime = 1
                 [set ValuePerCareer ValuePerCareer + [Value] of unit site]
                 [set ValuePerCareer precision ((ValuePerCareer * (OnUnitstime  - 1) + [Value] of unit site) / OnUnitstime) 2]
  ]
  ask humans with [site = -1] [set freetime freetime + 1]

  ask units with [PersonID != -1][set Busytime Busytime + 1]

 SetCoalitionObjectives
;; CoalitionDecisions        NO CODE NOW in the file!!!!!!!!!!!

Results

 if one-of humans with [site = -1] != nobody
     [ ask one-of humans with [site = -1] [fd random InfluenceRadius / 6] ]     ;; shift one-of free humans from their places (after unit's leaving)

tick

  if (ticks mod 1 = 0 )  [ MeanCalculation OAICalculation false]      ;; if Mean Harisma not calculated yet, OAIndex value equal to 0

if (ticks = LastTick + 1)
  [
   PersonsDoublingUnits UnitsDoublingPersons DecisionsbyLevels
   write "Results"
   write precision PotentialProductivity 2
   write precision mean ActualProductivityHistory 2

   show precision mean MissedProductivityHistory 2

   show precision mean [harisma] of humans 2
   show precision mean MeanActualProductivityHistory 2
   write precision mean MeanMissedProductivityHistory 2
   stop
  ] ;; Diagnostic and total statistics
end
;;////////  EndOfGO  //////////EndOfGO////////////  EndOfGO //////////EndOfGO////////  EndOfGO  /////////////////// EndOfGO


;; ---------------------------------------- SP1 (No Decisions on Level 2) ---------------------------------------------
to sp1


  if Initiative
      [
      Decision_EU_IB_FH              ;; free humans search for empty unit (with Boss)
      Decision_EU_IB_BH              ;; busy humans search for empty unit (with Boss) with value > current unit value
      ]
       ask one-of units [ D_VoluntaryDismisial  who ]  ;;

                  ask units with [level = 2 and personID != -1]
                  [
                       ;; No Decisions on Level 2 !!!!!!!!!!!!!!!!!!!
                       ;; ifelse OAIndex < GradeH
                       ;; [D_GetGoodSuborder4L3 who D_GetAnySuborder4L3 who]
                       ;; [D_ReplaceSuborder4L3onFreeHuman who]]

                  ]
                  ask units with [level = 1 and personID != -1]
                  [
                      ifelse OAIndex < GradeH
                         [D_DownGradeWorstBoss who]
                         [D_UpGradeBestBoss who]
                  ]

                  ask units with [level = 0 and personID != -1]
                  [
                     ifelse  OAIndex <= GradeH
                     [D_DownGradeWorstBoss who D_UpGradeBestBoss who]
                     [D_UpGradeBestBoss who] ;; else
                  ]

end

;; ---------------------------------------- SP2 (3-points control Behaviour Screenplay) ------------------------
to sp2 ;; First Edition as Etalon )))


  if Initiative
      [
       Decision_EU_IB_FH              ;; free humans search for empty unit (with Boss)
       Decision_EU_IB_BH              ;; busy humans search for empty unit (with Boss) with value > current unit value
      ]
                  ask one-of units [ D_VoluntaryDismisial  who ]  ;;
                  ask units with [level = 2]
                  [
                       if GradeM <= OAIndex and OAIndex < GradeH [D_Pass2NoBossUnit who]
                       if GradeL <= OAIndex and OAIndex < GradeM
                        [ifelse PerceptON [D_Pass2FrendLev3 who] [D_DownGradeOrDismiss who]]

                       if OAIndex < GradeL [D_DismissWorst who]
                       D_GetGoodSuborder4L3 who
                  ]
                  ask units with [level = 1]
                  [
                       if GradeM  <= OAIndex and OAIndex < GradeH
                        [ifelse PerceptON [D_Pass2FrendSameLevel who] [D_DownGradeOrDismiss who]]
                       if GradeL <= OAIndex and OAIndex < GradeM  [D_DismissWorst who]
                       if OAIndex < GradeL [D_DismissWorst who]
                       D_UpGradeBestBoss who
                  ]

                  ask units with [level = 0]
                  [D_DownGradeOrDismiss who D_UpGradeBestBoss who]

end
;; ---------------------------------------- SP3 (No Decisions on Level 0)------------------------
to sp3

   if Initiative
      [
      Decision_EU_IB_FH              ;; free humans search for empty unit (with Boss)
      Decision_EU_IB_BH              ;; busy humans search for empty unit (with Boss) with value > current unit value
      ]
       ask one-of units [ D_VoluntaryDismisial  who ]  ;;

                  ask units with [level = 2 and personID != -1]
                  [
                       if GradeM <= OAIndex and OAIndex < GradeH [D_Pass2NoBossUnit who]
                       if GradeL <= OAIndex and OAIndex < GradeM
                        [ifelse PerceptON [D_Pass2FrendLev3 who] [D_DownGradeOrDismiss who]]

                       if OAIndex < GradeL [D_DismissWorst who]
                       D_GetGoodSuborder4L3 who
                  ]

                  ask units with [level = 1 and personID != -1]
                  [
                       ifelse OAIndex < GradeH
                         [D_DownGradeWorstBoss who  D_UpGradeBestBoss who]
                         [D_UpGradeBestBoss who]

                  ]

                  ask units with [level = 0 and personID != -1]
                  [
                     ;; No Decisions on Level 0 !!!!!!!!!!!!!!!!!!!
                     ;; ifelse  OAIndex <= GradeH
                     ;; [D_DownGradeWorstBoss who D_UpGradeBestBoss who]
                     ;; [D_UpGradeBestBoss who] ;; else
                  ]

end
;; ---------------------------------------- SP4 (Random All decisions Behaviour Screenplay)------------------------
to sp4

   if Initiative
      [
      Decision_EU_IB_FH              ;; free humans search for empty unit (with Boss)
      Decision_EU_IB_BH              ;; busy humans search for empty unit (with Boss) with value > current unit value
      ]
       ask one-of units [ D_VoluntaryDismisial  who ]  ;;

       let rndvalue random 100

                  ask units with [level < 3 and personID != -1]
                  [
                   ifelse OAIndex < GradeH

                       [(
                         ifelse rndvalue < 10 [D_Pass2NoBossUnit who]
                                rndvalue < 20 [D_Pass2FrendLev3 who]
                                rndvalue < 30 [D_Pass2FrendSameLevel who]
                                rndvalue < 40 [D_GetAnySuborder4L3 who]
                                rndvalue < 50 [D_GetGoodSuborder4L3 who]
                                rndvalue < 60 [D_ReplaceSuborder4L3onFreeHuman who]
                                rndvalue < 70 [D_DownGradeOrDismiss who]
                                rndvalue < 80 [D_DownGradeWorstBoss who]
                                rndvalue < 90 [D_UpGradeBestBoss who]
                               [D_DismissWorst who]
                       )
                       ]
                       []
                  ]


end
;; ---------------------------------------- SP5 (No control!!!!!!!!!!! behaviour Screenplay)------------------------
to sp5

   if Initiative
      [
      Decision_EU_IB_FH              ;; free humans search for empty unit (with Boss)
      Decision_EU_IB_BH              ;; busy humans search for empty unit (with Boss) with value > current unit value
      ]
       ask one-of units [ D_VoluntaryDismisial  who ]  ;;

end
;; ==================================================================================================================================
;; HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH-------    DECISIONS      -------HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;; ==================================================================================================================================

;; ------------------------------------------------- Empty Unit W I T H Boss, Free Human (as applicant) ---------------------------------------------------
to Decision_EU_IB_FH  ;; Empty Unit WITH Boss (there Is a boss), Free Human (as applicant). Run for not Top manager's Unit !!!!!!
  ;;  -------------------------------------------------------------------------------------------
  ;;  ----- This is an offer as a (free) candidate for the vacancy of the Boss
  ;;  ----- So there is Boss at empty unit, his approval required.
  ;;  ----- Ceiteria: is positive Perception by the Boss of the applicant
  ;;  -------------------------------------------------------------------------------------------
  if one-of units with [personID = -1 and my_boss != -1 and [PersonID] of unit my_boss != -1] != nobody
  [
    ask one-of units with [personID = -1 and my_boss != -1 and [PersonID] of unit my_boss != -1] ;; -----there is empy, not Top manager, unit, WITH boss (person on boss position)
    [set v who set xx xcor set yy ycor
     let person_who -1                   ;; no one candidate person for the unit yet...
     let boss_who [PersonID] of unit my_boss
     if one-of humans with [site = -1 and member? v MyObjectives_list and ((not member? v history) or (not OneTimeAsgn))] != nobody
     [
      ask one-of humans with [site = -1 and member? v MyObjectives_list and ((not member? v history) or (not OneTimeAsgn))]  ;; there is free human (as applicant), so empty unit has level = 3 (as objectives for free humans)
      [
          let human_who who
          if ([item  human_who PerceptOfOther] of human boss_who >= 0) or not PerceptON    ;; Perceptions have matter
          ;; ----- CONDITION for assign human to unit:
          ;; ----- Boss of empty unit has positive Perception of the applicant. Boss agrees assign the human to the unit,
          ;; ----- unit is objective for the human (member of his MyObjectives_list).
          [
          set person_who who
          set site v set xcor xx set ycor yy set history lput "FHSrch" history set history lput v history  ;; person assigned to unit, update human's attributes
          ]
      ]
     ]

      ifelse person_who != -1           ;; person assigned to unit, update unit's attributes
        [
         set personID person_who set history_list lput "FHSrch" history_list set history_list lput person_who history_list
         set LastChangeTick ticks set ChangesQty ChangesQty + 1
         set PB1 PB1 + 1
         if LogWrite [write personID write " Decision_EU_IB_FH. Assign to Level " show [level] of unit v]
        ]
        [set PB0 PB0 + 1]

    ]
  ]

 end ;; ////////////////////////////////////////////////////////////////////////////////////////////////////////

;; ------------------------------------------------- Empty Unit W I T H Boss, Busy Human (as applicant) ---------------------------------------------------
to Decision_EU_IB_BH  ;; Empty Unit WITH Boss, Busy Human (as applicant). Run for not Top manager's Unit !!!!!!
  ;; -------------------------------------------------------------------------------------------
  ;; ----- This is attempt of Human's MIGRATION to unit (of 1,2 or 3 levels) with Value more then current.
  ;; ----- So there is Boss at empty unit, his approval required.
  ;; ----- Ceiteria: positive Perception by the Boss of the applicant
  ;; -------------------------------------------------------------------------------------------
  if one-of units with [personID = -1 and my_boss != -1 and [PersonID] of unit my_boss != -1] != nobody
  [
    ask one-of units with [personID = -1 and my_boss != -1 and [PersonID] of unit my_boss != -1] ;; -----there is empy, not Top manager, unit, WITH boss (person on boss position)
    [set v who set xx xcor set yy ycor
     let person_who -1                   ;; no one candidate person for the unit yet...
     let boss_who [PersonID] of unit my_boss
      if one-of humans with [site != -1 and member? v MyObjectives_list and [value] of unit site < [value] of unit v] != nobody
     [
      ask one-of humans with [site != -1 and member? v MyObjectives_list and [value] of unit site < [value] of unit v]  ;; there is Busy Human (from other Unit) (as applicant)
      [
          let human_who who
          if ([item  human_who PerceptOfOther] of human boss_who >= 0) or not PerceptON    ;; Perceptions have matter
          ;; ----- CONDITION for assign human to unit:
          ;; ----- Boss of empty unit has positive Perception of the applicant. Boss agrees assign him to unit,
          ;; ----- unit is objective for the human (member of his MyObjectives_list).
          [
          ;; It's need to free old unit from removed human
          ask unit site [set personID -1]
          ;; Record new values to human's attributes
          set site v set xcor xx set ycor yy set history lput "BHSrch" history set history lput v history
          ;; and store who of the human
          set person_who who
          ]
      ]
     ]

      ifelse person_who != -1           ;; person assigned to unit, update unit's attributes
        [
         set personID person_who set history_list lput "BHSrch" history_list set history_list lput person_who history_list
         ;; write "Decision_EUIBBH unit " show v
         set LastChangeTick ticks set ChangesQty ChangesQty + 1
         set PB2 PB2 + 1
        ]
        [set PB0 PB0 + 1]

    ]
  ]

 end ;; ////////////////////////////////////////////////////////////////////////////////////////////////////////

;; ------------------------------------------------- Human, plased at some unit, Voluntary Dismiss from the Unit ---------------------------------------------------
to D_VoluntaryDismisial  [PersonUnitWho] ;; Human, plased at some unit, Voluntary Dismiss from the Unit
  ;;  -------------------------------------------------------------------------------------------
  ;;  ----- This is Human's VoluntaryDismisial from current unit
  ;;  ----- Condition 1 (if PerceptON)
  ;;  ----- Condition 2 (if NO PerceptON)
  ;;  -------------------------------------------------------------------------------------------

  ;; ifelse  (unit PersonUnitWho != nobody and ([personID] of unit PersonUnitWho) != -1)
  ifelse PerceptON
  [
  ifelse one-of units with [who = PersonUnitWho and PersonID != -1] != nobody
  [ask unit PersonUnitWho
     [let Condition false            ;; some Condition of Dismission
      set xx xcor set yy ycor let LPersonWho [PersonID] of unit PersonUnitWho

     ShowHumansList PersonUnitWho false ;; to set HumanList (global var) to neighbor's (to the unit) who

     let li 0 let ls 0
     while [li < length HumanList]
      [ let lOtherpersonWho item li HumanList
        if  lOtherpersonWho > -1 [set ls ls + item lOtherpersonWho [PerceptOfOther] of human LPersonWho ]    ;; summ My perception of other humans (lpersonWho > -1) at neighbor's positions
        if LogWrite [ write ls ]
        set li li + 1
      ]

      if ls < 0 [set Condition true]  ;; sum of my perception of other at neighbor's positions

      ifelse Condition ;; Dismiss Human (from the Unit) and Vacating the unit
      [
      ask human  PersonID        ;; Dismiss Human (from the Unit)
        [ set site -1 set xcor xx + InfluenceRadius + 2 set ycor yy + 5 set history lput "VltDsm" history set history lput -1 history if LogWrite [write "Voluntary Dismissal " show who]] ;; Record new values to human's attributes
        ;; set xcor xx + InfluenceRadius + 2   IS FOR DISTANCE FROM "BAD" UNIT (IN SHORT TERM)
      ask unit PersonUnitWho     ;; Vacating the unit
        [ set personID -1 set D_list lput "VltDsm" D_list set D_list lput ticks D_list ]

      set LastChangeTick ticks set ChangesQty ChangesQty + 1
      set VDm VDm + 1
      ]
      [if LogWrite [write "VoluntaryDism. Dismissial condition is false. Unit, Human " write PersonUnitWho show [personID] of unit PersonUnitWho]]

     ]
  ]
  [if LogWrite [write PersonUnitWho show "VoluntaryDism. Error: There is not Unit or hasn't the person at it"]]
  ] ;; if PerceptON
  [;; else PerceptON
    ifelse one-of units with [who = PersonUnitWho and PersonID != -1] != nobody
     [ask unit PersonUnitWho
        [ ;;; NEED CODE NEED CODE NEED CODE NEED CODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        ]
     ]
     [if LogWrite [write PersonUnitWho show "VoluntaryDism. Error: There is not Unit or hasn't the person at it"]]
  ];; else PerceptON

 end ;; ////////////////////////////////////////// Human, plased at some unit, Voluntary Dismiss from the Unit //////////////////////////////////////

;; ------------------------------------------------- Boss (Human), plased at some unit, dismiss all persons from subordered Units ---------------------------------------------------
to D_DismissAll  [BossWho] ;; Boss (Human), plased at some unit (at any Level), dismiss all persons from subordered Units

 ;; .......................................
 ;; Dismis ALL suborders from NEXT low Level !!!           For ex: for Level 1 Unit dismiss it's suborders of Level 2.
 ;; Criteria: NO, unconditional action
 ;; .......................................

  ;; ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse one-of units with [who = BossWho and PersonID != -1] != nobody
  [
    ask unit BossWho
    [
      set v who
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody
      [
      ask units with [my_boss = v and PersonID != -1]
      [
      set xx xcor set yy ycor
      ask human  PersonID  ;; Dismiss Human (from subordered Unit)
          [
            set site -1 set xcor xx + InfluenceRadius + 2 set ycor yy + 5 set history lput "DmAll" history set history lput -1 history if LogWrite [write "Dismiss(All) " show who] ;; Record new values to human's attributes
          ]
          ;; It's need to free unit from removed human
          set personID -1
      ]
      set LastChangeTick ticks set ChangesQty ChangesQty + 1
      set DmAll DmAll + 1
      set D_list lput "DmAll" D_list ;; D_list of unit BossWho
      ]
      [if LogWrite [write "DismAll. No suborders for unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show "DismAll. Error: There  is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of D_DismissAll //////////////////////////////////////////////////////////////

 ;; ------------------------------------------------- Boss (Human), plased at some unit, dismiss the worst person from subordered Units ---------------------------------------------------
to D_DismissWorst  [BossWho] ;; Boss (Human), plased at some unit (at any Level), dismiss the worst person from subordered Units
  ;; Criteria min OAIndex

  ;; ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse one-of units with [who = BossWho and PersonID != -1] != nobody
  [
    ask unit BossWho     ;; BossWho here is number of unit, where the Boss (decision maker) now placed
     [
      set v who
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody
      [
         let localunit min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit    ;; the human with worst OAIndex from BossWho suborders

           ask human  person_who  ;; Dismiss Human (from subordered Unit)
            [set site -1 set xcor xx + InfluenceRadius + 2 set ycor yy + InfluenceRadius + 2 set history lput "DmWrst" history set history lput -1 history if LogWrite [write "Dismiss (Worst) " show who]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human
           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set DmWorst DmWorst + 1
          ask unit v [set D_list lput "DmWrst" D_list ]    ;; create-link-to human person_who [set birthdate ticks]
       ]
        [if LogWrite [write "DismWorst. No suborders for unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show " DismWorst. Error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of D_DismissWorst //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), plased at some unit, pass worst person from suborders to unit, which has't the Boss ------
to D_Pass2NoBossUnit [BossWho] ;; Boss (Human), plased at some unit, pass worst person from suborders to unit (at the same level as current suborder's one), which has't the Boss
;; -----------------------------
;; ----- Pass person to the same Level as it placed now
;; ----- If Boss of L1 pass his suborder (Boss of L2) by this way, it's pass the suborder to other Boss of L1
;; ----- Criteria: min OAIndex
;; -----------------------------
  ;;ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse one-of units with [who = BossWho and PersonID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who let llevel level
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody
      [
         let localunit min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit         ;; the worst people from BossWho suborders
         let new_unit -1   ;; for new unit who

         ifelse one-of units with [level = llevel + 1 and PersonID = -1 and [PersonId] of unit my_boss = -1] != nobody    ;; with [level = 3
         [                         ;;There is Empty unit with no Boss at Level 3
           ask one-of units with [level = llevel + 1 and PersonID = -1 and [PersonId] of unit my_boss = -1]               ;; with [level = 3
           [
           set new_unit who set xx xcor set yy ycor
           set PersonID person_who ;; Assign person to new unit
           set history_list lput "PsNBU" history_list set history_list lput person_who history_list ;; Record new values to unit's attributes
           ]

           ask human  person_who  ;; Record new values to human's attributes
            [set site new_unit set xcor xx set ycor yy set history lput "PsNBU" history set history lput new_unit history if LogWrite [write "Pass human " write person_who write " to No Boss Unit " show new_unit]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human
           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set P2NBU P2NBU + 1
           ask unit v [set D_list lput "PsNBU" D_list ]
          ] [if LogWrite [write "P2NoBossUnit. No empty unit w/o boss at Level " show llevel + 1]]
       ]
        [if LogWrite [write "P2NoBossUnit. No suborders for unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show "P2NoBossUnit. Error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of D_Pass2NoBossUnit //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), plased at some unit, pass worst person from suborders to unit, which has the negative percepted by him other Boss (on the same level) ------
to D_Pass2FrendLev3  [BossWho]  ;; Boss (Human), plased at some unit, pass worst person from suborders to unit, which has the negative percepted by him other Boss (on the same level)
;; -----------------------------
;; ----- Pass person to unit at Level 3
;; ----- If Boss of L1 pass his suborder (Boss of L2) by this way, it's downgrade for the suborder
;; ----- Criteria: min OAIndex
;; -----------------------------
  ;;ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse one-of units with [who = BossWho and PersonID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
        set v who let llevel level let lbossunitwho who let lpersonboss personID let lperceplist [PerceptOfOther] of human personID
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody
      [
         let localunit min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit         ;; the worst people from BossWho suborders
         let new_unit -1   ;; for new unit who

          ifelse one-of units with [level = 3 and PersonID = -1 and [PersonId] of unit my_boss != -1 and (item ([PersonId] of unit my_boss) lperceplist < 0)] != nobody
         [                         ;;There is Empty unit with Boss, who (Boss) is negatively perceived by me
           ask one-of units with [level = 3 and PersonID = -1 and [PersonId] of unit my_boss != -1 and (item ([PersonId] of unit my_boss) lperceplist < 0)]
           [  ;; there is empy unit with Boss, who (Boss) is negatively perceived by me


           set new_unit who set xx xcor set yy ycor
           set PersonID person_who                       ;; Assign person to new unit
           set history_list lput "PsFrd3" history_list set history_list lput person_who history_list ;; Record new values to unit's attributes
           ]

           ask human  person_who  ;; Record new values to human's attributes
            [set site new_unit set xcor xx set ycor yy set history lput "PsFrd3" history set history lput new_unit history if LogWrite [write "Boss " write lpersonboss write " Pass human " write person_who write " to Friend Boss's subordered Unit (L3) " show new_unit]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human
           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set P2Frnd3 P2Frnd3 + 1
           ask unit v [set D_list lput "PsFrd3" D_list ]
          ] [if LogWrite [show "Pass2FrndL3. No empty unit with negatively perceived by me boss at Level 3"]]
       ]
        [if LogWrite [write "P2FriendL3. No suborders for unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show "P2FriendL3. Error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of D_Pass2Frend //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), plased at some unit, pass worst person from suborders to unit, which has the negative percepted by him other Boss (on the same level) ------
to D_Pass2FrendSameLevel [BossWho] ;; Boss (Human), plased at some unit, pass worst person from suborders to unit, which has the negative percepted by him other Boss (on the same level)
;; -----------------------------
;; ----- Pass person to the same Level as it placed now
;; ----- If Boss of L1 pass his suborder (Boss of L2) by this way, it's pass the suborder to other Boss of L1
;; ----- Criteria: min OAIndex
;; -----------------------------
  ;;ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse one-of units with [who = BossWho and PersonID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
        set v who let llevel level let lbossunitwho who let lpersonboss personID let lperceplist [PerceptOfOther] of human personID
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody
      [
         let localunit min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit         ;; the worst people from BossWho suborders
         let new_unit -1   ;; for new unit who

          ifelse one-of units with [level = llevel + 1 and PersonID = -1 and [PersonId] of unit my_boss != -1 and (item ([PersonId] of unit my_boss) lperceplist < 0)] != nobody
         [                         ;;There is Empty unit with Boss, who (Boss) is negatively perceived by me
           ask one-of units with [level = llevel + 1 and PersonID = -1 and [PersonId] of unit my_boss != -1 and (item ([PersonId] of unit my_boss) lperceplist < 0)]
           [  ;; there is empy unit with Boss, who (Boss) is negatively perceived by me


           set new_unit who set xx xcor set yy ycor
           set PersonID person_who                       ;; Assign person to new unit
           set history_list lput "PsFrdSL" history_list set history_list lput person_who history_list ;; Record new values to unit's attributes
           ]

           ask human  person_who  ;; Record new values to human's attributes
            [set site new_unit set xcor xx set ycor yy set history lput "PsFrdSL" history set history lput new_unit history if LogWrite [write "Boss " write lpersonboss write " Pass human " write person_who write " to Friend Boss's subordered Unit " show new_unit]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human
           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set P2Frnd P2Frnd + 1
           ask unit v [set D_list lput "PsFrdSL" D_list ]
          ] [if LogWrite [write "P2FriendSameLev. No empty unit with negatively perceived by me boss at Level" show llevel + 1]]
       ]
        [if LogWrite [write "P2FriendSameLev. No suborders for unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show  "P2FriendSameLev. Error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of Pass2FrendSameLevel //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), DownGrade person at unit with OAI = 0 (then has no suborders) ------
to D_DownGradeOrDismiss [BossWho]  ;;
;; -----------------------------
;; ----- For Calling (by BossWho) from Level 0 and Level 1 units
;; ----- Pass person to the next down Level then it placed now, OR if no empty units for him, dismiss him
;; ----- Criteria: he has no suborders now (ONLY!!!! no perception conditions, coalitions etc.)
;; -----------------------------
  ;; ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse  one-of units with [who = BossWho  and personID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who
      let lpersonboss personID

      ifelse one-of units with [my_boss = v and PersonID != -1 and OAIndex = 0] != nobody
      [
         let localunit one-of units with [my_boss = v and PersonID != -1 and OAIndex = 0]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit    ;; people from BossWho suborders, with no suborders (OAIndex = 0)
         let llevel [level] of localunit

         let new_unit -1   ;; for new unit who. Local variable exists only within the enclosing block: "ifelse one-of units with... [...]"

         ifelse one-of units with [level = llevel + 1 and PersonID = -1 ] != nobody ;; search empy unit at next  down level for people with no suborders (OAIndex = 0)
         [
           ask one-of units with [level = llevel + 1 and PersonID = -1 ]
           [
           set new_unit who set xx xcor set yy ycor
           set PersonID person_who                       ;; Assign person (from level 2) to new unit (at level 3)
           set history_list lput person_who history_list ;; Record new values to unit's attributes
           ]

           ask human  person_who  ;; Record new values to human's attributes
            [set site new_unit set xcor xx set ycor yy set history lput "DWG/dm" history set history lput new_unit history if LogWrite [write "Boss " write lpersonboss write " DownGradeManager " write person_who write " to Unit " show new_unit]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human

           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set DwnGr DwnGr + 1
           ask unit v [set D_list lput "DWG/dm" D_list ]
         ]
          [if LogWrite [write "DownGradeManager. No empty unit for downgrade at Level" show llevel + 1]
          ;; No empty unit for downgrade person to one Level. So, we dismiss him )))
           ask human  person_who  ;; Record new values to human's attributes
            [set site -1 set xcor xx + InfluenceRadius + 2 set ycor yy + 5 set history lput "dwg/DM" history set history lput -1 history if LogWrite [write "Boss " write lpersonboss write " Dismiss " write person_who]] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human

           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set DwnGrDism DwnGrDism + 1
           ask unit v [set D_list lput "dwg/DM" D_list ]
         ]

      ]
        [if LogWrite [write "OAI of my suborders > 0 " show v]]
    ]
  ]
  [if LogWrite [write BossWho show  "DownGradeManager error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of ... //////////////////////////////////////////////////////////////
; ------------------------------------------ Boss (Human), DownGrade person at unit with OAI = 0 (then has no suborders) ------
to D_DownGradeWorstBoss [BossWho]  ;;
;; -----------------------------
;; ----- For Calling (by BossWho) from Level 0 and Level 1 units
;; ----- Pass sub ordered person to the next down Level then it placed now, if there is empty unit, or do nothig
;; ----- Criteria: min OAIindex (ONLY!!!! no perception conditions, coalitions etc.)
;; -----------------------------
  ;; ifelse  (unit BossWho != nobody and ([personID] of unit BossWho) != -1)
  ifelse  one-of units with [who = BossWho  and personID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who
      let lpersonboss personID

      ifelse min-one-of units with [my_boss = v and PersonID != -1] [OAIndex] != nobody  ;; there is humans as suborders
        ;; min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
      [
         let localunit min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit_who [who] of localunit
         let person_who [PersonID] of localunit    ;; people from BossWho suborders, with min OAIndex
         let llevel [level] of localunit

         let new_unit -1   ;; for new unit who. Local variable exists only within the enclosing block: "min-one-of units with ... [...]"

         ifelse one-of units with [level = llevel + 1 and PersonID = -1 ] != nobody ;; search empy unit at next  down level for people with no suborders (OAIndex = 0)
         [
           ask  one-of units with [level = llevel + 1 and PersonID = -1 ]
           [
           set new_unit who set xx xcor set yy ycor
           set PersonID person_who                       ;; Assign person (from level 2) to new unit (at level 3)
           set history_list lput person_who history_list ;; Record new values to unit's attributes
           ]

           ask human  person_who  ;; Record new values to human's attributes
            [set site new_unit set xcor xx set ycor yy set history lput "DwnWB" history set history lput new_unit history
             if LogWrite
              [
                write "Boss " write lpersonboss write " DownGradeManager " write person_who write " to Unit " write new_unit show [level] of unit new_unit
              ]
            ] ;; Record new values to human's attributes
           ask unit unit_who
            [set personID -1] ;; It's need to free unit from removed human

           set LastChangeTick ticks set ChangesQty ChangesQty + 1
           set DwnGr DwnGr + 1
           ask unit v [set D_list lput "DwnWB" D_list ]
         ]
         [
            if LogWrite [write "DownGradeWorstBoss. No empty unit at sub Level. No deal"]
         ]

      ]
        [if LogWrite [write "OAI of my suborders > 0 " show v]]
    ]
  ]
  [if LogWrite [write BossWho show  "DownGradeManager error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of D_DownGradeWorstBoss //////////////////////////////////////////////////////////////
; ------------------------------------------ Boss (Human), UpGrade person to unit with OAI = 0 (then has no suborders) ------
to D_UpGradeBestBoss [BossWho]     ;;
;; -----------------------------
;; ----- For Calling from Level 0 and Level 1 units
;; ----- Pass person to the next up Level then it is now
;; ----- Criteria: max (on the level) OAIndex  (ONLY!!!! no perception conditions, coalitions etc.)
;; -----------------------------
  ifelse  one-of units with [who = BossWho  and personID != -1] != nobody
  [
    ask unit BossWho   ;; BossWho here is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who let lpersonboss personID

      ifelse one-of units with [my_boss = v and PersonID = -1] != nobody     ;; there is empty unit for suborder (Destination Unit)
      [
         let unit2UP one-of units with [my_boss = v and PersonID = -1]
         let unit2UPWho [who] of unit2UP
         let llevel [level] of unit2UP

         let unit2FreeWho -1   ;; for old unit who. Local variable exists only within the enclosing block: "ifelse one-of units with [my_boss = v ...]"
         let PersonID2UP -1

          let unit2Free max-one-of units with [level = llevel + 1 and PersonID != -1 and ((not member? unit2UPWho  [history] of human PersonID) or (not OneTimeAsgn))] [OAIndex] ;; search  unit at next down level with max OAIndex

          ifelse unit2Free != nobody ;; it is!
         [
           ask unit2Free [set unit2FreeWho who set PersonID2UP PersonID ]

           ask unit unit2UPWho
           [
           set xx xcor set yy ycor
           set PersonID PersonID2UP                       ;; Free unit
           set history_list lput "UpGr" history_list set history_list lput PersonID2UP history_list ;; Record new values to unit's attributes
           ]

           ask human  PersonID2UP  ;; Record new values to human's attributes
            [set site unit2UPWho set xcor xx set ycor yy set history lput "UpGr" history set history lput unit2UPWho history
              if LogWrite [
                          write "Boss " write lpersonboss write " Upgrade manager " write PersonID2UP write " to Unit " show unit2UPWho
                          ]
            ] ;; Record new values to human's attributes
           ask unit unit2FreeWho
            [set personID -1] ;; It's need to free unit from removed human

           set LastChangeTick ticks set ChangesQty ChangesQty + 1   ;; statistics
           set UpGr UpGr + 1
           ask unit v [set D_list lput "UpGr" D_list]
         ]
          [if LogWrite [write "UpGradeManager. No person for upgrade unit " show unit2UPWho]]
      ]
        [if LogWrite [write "UpGradeManager. No empty units in my suborders. Unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show  "UpGrade error: There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of UpGrade //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), Search ANY person to the empty unit at Level 3 ------
to D_GetAnySuborder4L3 [BossWho] ;;
;; -----------------------------
;; ----- For Calling from Level 2 units
;; ----- Search ANY person to the empty unit at Level 3
;; ----- Criteria: free human, No Percept, no Harisma, No MyObjectives list conditions
;; -----------------------------
  ifelse  one-of units with [who = BossWho  and personID != -1 and level = 2] != nobody
  [
    ask unit BossWho   ;; BossWho is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who let lboss personID
      ifelse one-of units with [my_boss = v and PersonID = -1] != nobody
      [
         let unit2UP one-of units with [my_boss = v and PersonID = -1]  ;; my subordered empy unit
         let unit2UPWho [who] of unit2UP
         let lperson -1        ;; for new human
         if one-of humans with [site = -1] != nobody
          [ask one-of humans with [site = -1] [set lperson who]  ;; we found free human
          ]

          ifelse (lperson != -1) and ((not member? unit2UPWho [history] of human lperson) or (not OneTimeAsgn))
          ;;  and (not member? unit2UPWho [history] of human lperson)  ;; OK, search done, we have the Candidate
                        ;; and he has't new unit (for assign) in his history
         [
           ask unit unit2UPWho            ;; set new person to empty unit
           [
           set xx xcor set yy ycor
           set PersonID lperson
           set history_list lput "GtAnySb" history_list
           set history_list lput lperson history_list ;; Record new values to unit's attributes
           ]

           ask human  lperson  ;; Record new values to human's attributes
            [set site unit2UPWho set xcor xx set ycor yy
               set history lput "GtAnySb" history
               set history lput unit2UPWho history
               if LogWrite [write "Boss " write lboss write " get suborder " write lperson write " to Unit "
                            show unit2UPWho]
            ] ;; Record new values to human's attributes

           set LastChangeTick ticks set ChangesQty ChangesQty + 1  ;; statistics
           set GetAnySub GetAnySub + 1
           ask unit v [set D_list lput "GtAnySb" D_list]
        ] ;; search of free human
          [if LogWrite [write "GetAnySuborder4L3. Not found the person for unit " show unit2UPWho]]
      ]  ;; search empty Unit for BossWho
        [if LogWrite [write "GetAnySuborder4L3. No empty units in my suborders. Unit " show v]]
    ] ;; search BossWho at Level 3
  ]
  [if LogWrite [write BossWho show  "GetAnySuborder4L3. There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of GetAnySuborder4L3 //////////////////////////////////////////////////////////////
; ------------------------------------------ Boss (Human), Search person to the empty unit at Level 3 ------
to D_GetGoodSuborder4L3 [BossWho] ;;
;; -----------------------------
;; ----- For Calling from Level 2 units (BossWho place)
;; ----- Search person to the empty unit at Level 3
;; ----- Criteria: free human with harisma > HarismaMean and Percept (of new Boss) = 1
;; ----- OR (if its't free human): Criteria: OAIndex > 1, Percept (of new Boss) = 1 and Percept New Boss of old Boss (if exist) -1 and
;; ----- and Perception Candidate of new Boss = 1 (equal to his assept to pass to new unit)
;; ----- Or: First and Second conditions, Or only First
;; -----------------------------
  ifelse  one-of units with [who = BossWho  and personID != -1 and level = 2] != nobody
  [
    ask unit BossWho   ;; BossWho is number of unit, where the Boss (decisionmaker) now placed
     [
      set v who let lboss personID let TypeofCandidate ""
      ifelse one-of units with [my_boss = v and PersonID = -1] != nobody
      [
         let unit2UP one-of units with [my_boss = v and PersonID = -1]  ;; my subordered empy unit
         let unit2UPWho [who] of unit2UP
         let unit2FreeWho -1   ;; for old unit who. Local variable exists only within the enclosing block: "ifelse one-of units with [my_boss = v ...]"
         let lperson -1        ;; for new human

          ;; Now we start search the Candidate

          ;; Search 1 ---------------------------
          if one-of humans with [site = -1 and harisma >  HarismaMean and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON)] != nobody
          [ask one-of humans with [site = -1 and harisma >  HarismaMean and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON)]
            [set lperson who set TypeofCandidate "Free human"] ;; we found free human
          ]
          ;; Search 2 ---------------------------
          if lperson = -1 and one-of humans with [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON) and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)] != nobody
          [ask one-of humans with                [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON) and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)]
            [
              if ([personID] of unit ([my_boss] of unit site) = -1) or ([personID] of unit ([my_boss] of unit site) != -1 and (item ([personID] of unit ([my_boss] of unit site)) [PerceptOfOther] of human lboss <= 0 or not PerceptON))
              [set lperson who set unit2FreeWho site set TypeofCandidate "Busy human, ambgty percept OK, bad old Boss or no Boss"]  ;; we NOT found free human, then we search for Busy human with ambiguity percept OK
            ] ;; and human has not the person on the old Boss unit or Percept New Boss of old Boss (if exist) -1
          ]
          ;; Search 3 ---------------------------
          if lperson = -1 and one-of humans with [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON) and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)] != nobody
          [ ask one-of humans with               [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON) and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)]
            [set lperson who set unit2FreeWho site set TypeofCandidate "Busy human, ambgty percept OK, Boss no matter"]  ;; we NOT found ...
          ] ;; ... then we search for Busy human with ambiguity percept OK, no matter who is him old Boss
          ;; Search 4 ---------------------------
          if lperson = -1 and one-of humans with [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)] != nobody
          [ask one-of humans with                [site != -1 and [level] of unit site = 3 and [OAIndex] of unit site > 1 and (item lboss [PerceptOfOther] of human who >= 1 or not PerceptON)]
             [set lperson who set unit2FreeWho site set TypeofCandidate "Busy human, New Boss percept of Candidate and Old Boss no matter"]  ;; we NOT found ...
          ] ;; ... then we search for Busy human with perception of New Boss >= 0, no matter how New Boss percept him and his old Boss (if exist)

          ifelse (lperson != -1) and (not member? unit2UPWho [history] of human lperson)  ;; OK, search done, we have the Candidate
          ;; and he has't new unit (for assign) in his history
         [
           ask unit unit2UPWho            ;; set new person to empty unit
           [
           set xx xcor set yy ycor
           set PersonID lperson

           ifelse TypeofCandidate = "Free human"
           [set history_list lput "GtFrSb" history_list] [set history_list lput "GtBsSb" history_list]
           set history_list lput lperson history_list ;; Record new values to unit's attributes
           ]

           ask human  lperson  ;; Record new values to human's attributes
            [set site unit2UPWho set xcor xx set ycor yy
             ifelse TypeofCandidate = "Free human"
              [set history lput "GtFrSb" history] [set history lput "GtBsSb" history]
               set history lput unit2UPWho history
               if LogWrite [write "Boss " write lboss write " get suborder " write lperson write " to Unit "
                            show unit2UPWho show TypeofCandidate]
            ] ;; Record new values to human's attributes

           if unit2FreeWho != -1
            [ask unit unit2FreeWho [set personID -1] ]   ;; It's need to free unit from removed human

           set LastChangeTick ticks set ChangesQty ChangesQty + 1  ;; statistics
           ifelse TypeofCandidate = "Free human"
            [   set GetFreSub GetFreSub + 1
                ask unit v [set D_list lput "GtFrSb" D_list]
            ]
            [   set GetSub GetSub + 1
                ask unit v [set D_list lput "GtBsSb" D_list]
            ]
         ]
          [if LogWrite [write "GetSuborder. Not found the person for unit " show unit2UPWho]]
      ]
        [if LogWrite [write "GetSuborder. No empty units in my suborders. Unit " show v]]
    ]
  ]
  [if LogWrite [write BossWho show  "GetSuborder. There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of GetSuborder //////////////////////////////////////////////////////////////

; ------------------------------------------ Boss (Human), Search person to the empty unit at Level 3 ------
to D_ReplaceSuborder4L3onFreeHuman [BossWho] ;;
;; -----------------------------
;; ----- For Calling from Level 2 units (BossWho place)
;; ----- Search person to the busy (by human with min Harisma of BossWho Suborders)  unit at Level 3
;; ----- Criteria: free human with harisma > min Harisma and Percept (of new Boss) = 1
;; -----------------------------
  ifelse  one-of units with [who = BossWho  and personID != -1 and level = 2] != nobody
  [
    ask unit BossWho   ;; BossWho is number of unit, where the Boss (decisionmaker) now placed
     [

      set v who let lboss personID
      ifelse one-of units with [my_boss = v and PersonID != -1] != nobody  ;; Boss has the suborder now
      [
         ;; Search for suborder with min OAIndex

         let unit2UP min-one-of units with [my_boss = v and PersonID != -1] [OAIndex]
         let unit2UPWho [who] of unit2UP
         let personOut_who [PersonID] of unit2UP    ;; the human with worst OAIndex from BossWho suborders
         let personMinHarisma [harisma] of human personOut_who
         let lpersonIn -1        ;; for new human

          ;; Now we start search the Candidate 4 In

          if one-of humans with [site = -1 and harisma > personMinHarisma and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON)] != nobody
          [ask one-of humans with [site = -1 and harisma > personMinHarisma and (item who [PerceptOfOther] of human lboss >= 1 or not PerceptON)]
            [set lpersonIn who] ;; we found free human 4 In

          ]


         ifelse (lpersonIn != -1) and (not member? unit2UPWho [history] of human lpersonIn)  ;; OK, search done, we have the Candidate
          ;; and he has't new unit (for assign) in his history
         [
           ask unit unit2UPWho            ;; set new person to unit unit2UPWho
           [
           set xx xcor set yy ycor
           set PersonID lpersonIn
           set history_list lput "RplSb" history_list
           set history_list lput lpersonIn history_list             ;; Record new values to unit's attributes
           ]

           ask human  lpersonIn  ;; Record new values to human's lpersonIn attributes
            [set site unit2UPWho set xcor xx set ycor yy
             set history lput "ReplIn" history
             set history lput unit2UPWho history

           ask unit v [set D_list lput "ReplSub" D_list]   ;; write decision to BossWho unit history

            if LogWrite
             [
                write "Boss " write lboss write " Replace suborder " write lpersonIn write " to Unit " show unit2UPWho
             ]
            ] ;; Record new values to human's attributes

            ask human personOut_who
            [
              set site -1 set xcor xcor + InfluenceRadius + 2 set ycor ycor + InfluenceRadius + 2
            ] ;; Record new values to human's personOut_who attributes

           set LastChangeTick ticks set ChangesQty ChangesQty + 1  ;; statistics
           set ReplSub ReplSub + 1


        ]
          [ if LogWrite [write "ReplaceSuborder. Not found the person for unit " show unit2UPWho]]
      ]
        [ if LogWrite   [write "ReplaceSuborder. No my suborders. Unit " show v]]
    ]
  ]
  [ if LogWrite         [write BossWho show  "ReplaceSuborder. There is not Unit or has't the person at it"]]
 end ;; ////////////////////////////////////////// of ReplaceSuborder //////////////////////////////////////////////////////////////

;;-------------------------------------------------- Show Decisions by Levels of DMakers  ----------------------------------------------------
to DecisionsbyLevels                          ;;
  show "---------------- DecisionsbyLevels "
  show "ZERO Level ..."
  ask units with [level = 0 ]
    [ ;; write D_list
      show length D_list
    ]
  show " ... FIRST ..."
  ask units with [level = 1 ]
    [ ;; write D_list
      show length D_list
    ]
  show "... and SECOND Levels"
  ask units with [level = 2 ]
    [ ;; write D_list
      show length D_list
    ]
end


;;-------------------------------------------------- Show Suborders  ----------------------------------------------------
to ShowSuborders                          ;;
  show "----------------"
  show "Suborders of Bosses ZERO ..."
  ask units with [level = 0 ]
    [ write OAIndex let l who write " Boss " show PersonID
      ask units with [my_boss = l] [write OAIndex show PersonID]
    ]
  show " ... FIRST ..."
  ask units with [level = 1]
    [ write OAIndex let l who write " Boss " show PersonID
      ask units with [my_boss = l] [write OAIndex show PersonID]
    ]
  show "... and SECOND Levels"
  ask units with [level = 2]
    [ write OAIndex let l who write " Boss " show PersonID
      ask units with [my_boss = l] [write OAIndex show PersonID]
    ]
end



;;-------------------------------------------------- Show Coalitions results  ----------------------------------------------------
to ShowCoalitionsResults
  show "---------------- Coalitions results -----"
  let color_list ["RED" "BLUE" "BLACK" "GREEN" "YELLOW"]
  ask Coalitions
  [ let lv who
    write "Coalition" write item (lv - HumansQty - UnitsQty ) color_list show lv
    show "Suborders of Bosses ZERO ..."
    ask units with [level = 0]
    [if personID != -1 and [coalitionID] of human personID = lv
      [ write OAIndex let l who write " Boss " show PersonID
        ask units with [my_boss = l] [write OAIndex show PersonID]
      ]
    ]
    show "... FIRST ..."
    ask units with [level = 1]
    [if personID != -1 and [coalitionID] of human personID = lv
      [ write OAIndex let l who write " Boss " show PersonID
        ask units with [my_boss = l] [write OAIndex show PersonID]
      ]
    ]
    show "... SECOND ..."
    ask units with [level = 2]
    [if personID != -1 and [coalitionID] of human personID = lv
      [ write OAIndex let l who write " Boss " show PersonID
        ask units with [my_boss = l] [write OAIndex show PersonID]
      ]
    ]
    show "... and THIRD Levels"
    ask units with [level = 3]
    [if personID != -1 and [coalitionID] of human personID = lv
      [ write OAIndex let l who write " Suborder " show PersonID
        ask units with [my_boss = l] [write OAIndex show PersonID]
      ]
    ]
    show "and Free humans"
    ask humans with [site = -1 and coalitionID = lv]
         [ write "Free " show who]
 ]
end

;;-------------------------------------------------- PerceptSum4UnitCalc  ----------------------------------------------------
to PerceptSum4UnitCalc     ;; Calculate Perception sum of other persons for the Human, assisgned to the Unit. If no human at unit sum = 0
                        ;; and write value to Unit (!!!) attribute
  ask units
  [ set u 0
    ifelse personID = -1
    [set PercptSumOfCurrHuman 0 set color originalcolor]
    [ let PersonOnUnitWho personID
      ;; My Boss and Colleagues
      if my_boss != -1    ;; for no top manager
      [ let MyBossWho my_boss
        ask unit my_boss
        [if personID != -1 [set v personID set u u + ([item v PerceptOfOther] of human PersonOnUnitWho)]] ;; add my perceptoin of Boss
        ask units with [my_boss = MyBossWho]
        [if personID != -1 [set v personID set u u + ([item v PerceptOfOther] of human PersonOnUnitWho)]] ;; and Colleagues
      ]
      ;; My Suborders
        ask units with [my_boss = PersonOnUnitWho] ;; Suborders
        [if personID != -1 [set v personID set u u + ([item v PerceptOfOther] of human PersonOnUnitWho)]]

        set PercptSumOfCurrHuman u ;; write value to Unit attribute
        (ifelse PercptSumOfCurrHuman < -2 [set color 14] PercptSumOfCurrHuman >  2 [set color 54]   [set color originalcolor])
    ] ;; else personID = -1
  ]
end
;;-------------------------------------------------- AssignAnyUnits  ----------------------------------------------------
to AssignOrderedPersOnUnits                          ;; Assign humans ordered by Who to All Units
  ask units [set personID -1]
  ask humans [set site -1]                            ;; min [who] of humans with [site = -1]
  while [(count units with [personID = -1] != 0) and (count humans with [site = -1] != 0)]
  [
      let uu min [who] of units with [personID = -1]
      ask unit uu
      [
      set vpx xcor set vpy ycor set v who
      let hh min [who] of humans with [site = -1]
      ask human hh  [set site v set xcor vpx set ycor vpy set vp who set history lput "AsOrdPers" history set history lput v history]
      set personID vp
      set history_list lput "AsAOrdPers" history_list  set history_list lput vp history_list
      ]
  ]
end

;;-------------------------------------------------- AssignAnyUnits  ----------------------------------------------------
to AssignAnyUnits                          ;; Assign humans to all level Units in according with FirstDistrDensity
  ask units with [personID = -1]
  [set vpx xcor set vpy ycor set v who
    if one-of humans with [site = -1] != nobody  and random-float 1 < FirstDistrDensity    ;; there is empty head's units and free persons
    [
      ask one-of humans with [site = -1] [set site v set xcor vpx set ycor vpy set vp who set history lput "AsAnyUn" history set history lput v history]
      set personID vp
      set history_list lput "AsAnyUn" history_list  set history_list lput vp history_list
    ]
  ]
end


;;-------------------------------------------------- AssignOnlySubs ----------------------------------------------------
to AssignOnlySubs                          ;; first persons to units distribution
  ask units with [personID = -1 and level = 3]
  [set vpx xcor set vpy ycor set v who
    if one-of humans with [site = -1] != nobody     ;; there is empty units and free persons
    [
      ask one-of humans with [site = -1] [set site v set xcor vpx set ycor vpy set vp who set history lput "AsAllSbrd" history set history lput v history]
      set personID vp
      set history_list lput "AsAllSbrd" history_list set history_list lput vp history_list
    ]
  ]
end


;;-------------------------------------------------- Assign Only Top ----------------------------------------------------
to AssignTop                          ;;  In first we assign human on unit with level 0
  ask units with [personID = -1 and level = 0]
  [set vpx xcor set vpy ycor set v who
    if one-of humans with [site = -1] != nobody      ;; there is empty head's units and free persons
    [
      ask one-of humans with [site = -1] [set site v set xcor vpx set ycor vpy set vp who set history lput "AsTopMgr" history set history lput v history]
      set personID vp
      set history_list lput "AsTopMgr" history_list set history_list lput vp history_list
    ]
  ]
end


;;-------------------------------------------------- Assign Heads----------------------------------------------------
to AssignHeads                          ;;  In first we assign humans to all (no matter for FirstDistrDensity) Head's Units...
  ask units with [personID = -1 and level < 3]
  [set vpx xcor set vpy ycor set v who
    if one-of humans with [site = -1] != nobody      ;; there is empty head's units and free persons
    [
      ask one-of humans with [site = -1] [set site v set xcor vpx set ycor vpy set vp who set history lput "AsHeads" history set history lput v history]
      set personID vp
      set history_list lput "AsHeads" history_list set history_list lput vp history_list
    ]
  ]
end


;;-------------------------------------------------- Assign Suborders ----------------------------------------------------
to AssignSuborders                          ;; ... and then (after AssignHeads) - assign humans to units with level = 3
  ask units with [personID = -1 and level = 3]
  [set vpx xcor set vpy ycor set v who
    if one-of humans with [site = -1] != nobody and random-float 1 < FirstDistrDensity     ;; there is empty units and free persons
    [
;;;;      if one-of humans with [site = -1] != nobody
;;;;      [
         ask one-of humans with [site = -1] [set site v set xcor vpx set ycor vpy set vp who set history lput "AsSbRnd" history set history lput v history]
;;;;      ]
      set personID vp
      set history_list lput "AsSbRnd" history_list set history_list lput vp history_list
    ]
  ]
end


;;-------------------------------------------------- SetCoalitions ----------------------------------------
to SetCoalitions
  let color_list [red blue black green yellow]
  ;; show color_list
  set u 0
  while [u < CoalitionsQty0]
  [
    create-ordered-coalitions 1 [set xcor max-pxcor set ycor max-pycor - 2 * u set color gray  set shape "flag" set v who set Objectives_list [] ]
    ask humans with [coalitionID = -1 and (who mod CoalitionsQty0) = u] [set coalitionID v  set color item u color_list]
    set u u + 1
  ]
  ask humans [ if random 100 < NoCoalition% [set coalitionID -1  set color white]  ]  ;;NoCoalition% is % out of coalition persons
end
;;-------------------------------------------------- SetHumans ----------------------------------------------------
to SetHumans
create-ordered-humans HumansQty
  [
    set size turt_size set shape "person"
    set label who
    setxy random-xcor random-ycor
    set harisma precision (max list 0 (random-normal 100 30)) 2                     ;;   2 precision (random 100) 2
    set color white
    set MyObjectives_list []
    set site -1
    set coalitionID -1
    set PerceptOfOther []
    set history []
    set onunitstime 0
    set onL3time 0
    set onL2time 0
    set onL1time 0
    set freetime 0
    set ValuePerCareer 0
  ]
end
;;----------------------------------------------------------------------------------------------------------

;; ------------------------------------------------- SetUnits ----------------------------------------------
to SetUnits
    create-ordered-units 1 ;; UnitsQty
    [
    set level 0
    set my_boss -1
    setxy 0 0
    set color 74
    set shape "star"
    set value 100 * (4 - level)
    set size turt_size
    set personID -1
    set label who
    set label-color black
    set history_list []
    set Busytime 0
    set D_list []
    let u1 who

    hatch Lev1Units [
            set level 1 set my_boss u1 set color color + 1 let u2 who set shape "pentagon" set value 100 * (4 - level) set label who ;;+ random 100 set label value
            hatch Lev2Units [
                    set level 2 set my_boss u2 set color color + 1 let u3 who set shape "square" set value 100 * (4 - level)  set label who
                    hatch Lev3Units [
                                    set level 3 set my_boss u3 set color color + 1 set shape "circle" set value 100 * (4 - level)  set label who
                                    ]
                            ]
                    ] ;; close 4 " hatch Lev1Units"
    ]
;;----------------- set topology of the hierarhy -------------------------------
  layout-circle units with [level = 1] L0to1Dist

  ask units  with [level = 1]
             [  let alfa 360 / (Lev1Units * Lev2Units) ;;   let beta 360 / Lev1Units         ;;let hamma 0
                set u who set xx xcor set yy ycor
                let x1 L1to2Dist * xx / L0to1Dist
                let y1 L1to2Dist * yy / L0to1Dist
                ifelse RegularPlace  ;; Regular units layout (in the World space) OR stochastic layout
                [
                ask units with [level = 2 and my_boss = u] [setxy ((xx + x1) * cos (alfa * (who mod Lev2Units)) + (yy + y1) * (- sin (alfa * (who mod Lev2Units)))) ((xx + x1) * sin (alfa * (who mod Lev2Units)) + (yy + y1) * cos (alfa * (who mod Lev2Units)))]
                ]
                [
                ask units with [level = 2 and my_boss = u] [setxy (xx + random L1to2Dist - L1to2Dist / 2) (yy + random L1to2Dist - L1to2Dist / 2)]
                ]
             ]

  ask units  with [level = 2]
             [  let alfa 360 / (Lev1Units * Lev2Units * Lev3Units)
                set u who set xx xcor set yy ycor
                let x1 L2to3Dist * xx / L1to2Dist
                let y1 L2to3Dist * yy / L1to2Dist
                ifelse RegularPlace   ;; Regular units layout (in the World space) OR stochastic layout
                [
                ask units with [level = 3 and my_boss = u] [setxy ((xx + x1) * cos (alfa * (who mod Lev3Units)) + (yy + y1) * (- sin (alfa * (who mod Lev3Units)))) ((xx + x1) * sin (alfa * (who mod Lev3Units)) + (yy + y1) * cos (alfa * (who mod Lev3Units)))]
                ]
                [
                ask units with [level = 3 and my_boss = u] [setxy (xx + random L2to3Dist - L2to3Dist / 2) (yy + random L2to3Dist - L2to3Dist / 2)]
                ]
             ]
;;-------------------------------------------------------------------------------
  set UnitsQty 1 + Lev1Units + Lev1Units * Lev2Units + Lev1Units * Lev2Units * Lev3Units
  ask units [set originalcolor color]
end



;;-------------------------------------------------- ShowSubset (Button) ------------------------------------
to ShowSubset
  let color_list [red blue black green yellow 35 25 75 85 95]

  if subset = "OriginalColor"  [ask units [set color originalcolor] ]
  if subset = "Heads"       [let i 0
                               ask units with [level = 1]
                               [set color item i color_list let j who
                                ask units  with [level = 2 and my_boss = j]
                                [ set color item i color_list let k who ]
                                set i i + 1
                               ]
                               ask units with [level = 0] [set color 127]
                             ]
  if subset = "Departments"  [let i 0
                              ask units with [level = 1]
                               [set color item i color_list let j who
                                ask units  with [level = 2 and my_boss = j]
                                [
                                 set color item i color_list let k who
                                 ask units  with [level = 3 and my_boss = k]
                                           [set color item i color_list]
                                ]
                                set i i + 1
                               ]
                             ]

 if subset = "Coal Objectives" and (one-of coalitions with [who = AgentWho] != nobody)    ;;; BAD CODE !!!!!!!!!!!!!!!
  [if length [Objectives_list] of coalition AgentWho > 0
  [let work_list [Objectives_list] of coalition AgentWho
   ask units
    [if member? who work_list [set color item 0 color_list] ]
  ]
  ]
 if subset = "Unit" and (one-of units with [who = AgentWho] != nobody)
  [ask unit AgentWho [set color item 0 color_list]]
end



;; -------------------------------------------------SetCoalitionObjectives-----------------------------------------
to SetCoalitionObjectives          ;;  if Coalitions exists then ...

  ask Coalitions
  [ set v who let work_list []
    ask humans with [coalitionID = v and site != -1]
    [set u site
     if [my_boss] of unit u != -1 [set work_list lput ([my_boss] of unit u) work_list]    ;; boss unit to coalitions objectives
     set work_list lput u work_list
    ]

    set Objectives_list remove-duplicates work_list ;; Not eliminates coalition's places from Objectives_list

  ]
end
;; -----------------------------------------------SetPersonalObjectives -------------------------------------------
to SetPersonalObjectives           ;;  the goal for the subordinate is the place of the boss

  ask humans with [ site != -1]
    [set u site set MyObjectives_list []
     if [my_boss] of unit u != -1 [set MyObjectives_list lput ([my_boss] of unit u) MyObjectives_list] ;; My_boss = -1 only for Top Manager
     ;; objectives (now) is only my boss'es place. It's possible set objectives for level i humans as all units at level i - 1
    ]
   ask humans with [ site = -1]
    [

      if one-of units with [level = 3] in-radius InfluenceRadius != nobody
          [set u [who] of one-of units with [level = 3] in-radius InfluenceRadius
           set MyObjectives_list lput u MyObjectives_list
          ]  ;; the target is the nearest place of the subordinate, for the person who does not currently occupy any position
      set MyObjectives_list remove-duplicates MyObjectives_list
    ]
end
;; ------------------------------------------------ HumanDoublingSites (Button) -------------------------------------
to PersonsDoublingUnits
let LNODoubling true
ask humans with [site != -1]
[let lv who let ls site let lc coalitionID
  ask humans with [who != lv]
  [if site = ls
      [ set LNODoubling false write "Human, site, coal " write lv write ls write lc write " Human, site, coal " write who write site show coalitionID ]
  ]
]
if LNODoubling = true [show "HumanDoublingSites. NO Doubling"]
end
;; ------------------------------------------------ UnitsDoublingPersons (Button) -----------------------------------
to UnitsDoublingPersons
let LNODoubling true
ask units with [personID != -1]
[let lv who let lh personID
  ask units with [who != lv]
  [if personID = lh
      [ set LNODoubling false write "Unit, human " write lv write lh write " Unit, human " write who show personID ]
  ]
]
if LNODoubling = true [show "UnitsDoublingPersons. NO Doubling"]
end
;; ------------------------------------------- ShowUnitsHistory ------------------------------------------
to ShowUnitsHistory
  ;; show who of persons assigned to units throw the simulation
  show "ShowUnitsHistory. HumanNow, UnitWho,  Busitime, history"
  show "Level 0 " ask units with [level = 0 ] [ write PersonID write Busytime show history_list]
  show "Level 1 " ask units with [level = 1 ] [ write PersonID write Busytime show history_list]
  show "Level 2 " ask units with [level = 2 ] [ write PersonID write Busytime show history_list]
  show "Level 3 " ask units with [level = 3 ] [ write PersonID write Busytime show history_list]

  ;;show "Level 1 Busy units " ask units with [level = 1 and personID != -1] [ write Busytime show history_list]
;;  show "Now free units" ask units with [level = 1 and personID = -1] [ write Busytime show history_list]
;; show "Level 2 Busy units " ask units with [level = 2 and personID != -1] [ write Busytime show history_list]
 ;; show "Now free units" ask units with [level = 2 and personID = -1] [ write Busytime show history_list]
 ;; show "Level 3 Busy units " ask units with [level = 3 and personID != -1] [ write Busytime show history_list]
 ;; show "Now free units" ask units with [level = 3 and personID = -1] [ write Busytime show history_list]
end
;; ------------------------------------------- ShowHumansHistory (Button) ------------------------------------------
to ShowHumansHistory
  ;; show who of units, person was assigned to throw the simulation
  show "Humans history. Who, Harisma, Time on units, Mean Value on Units, History with Units ID" show "On units"
  show  "Level 0 " ask units with [level = 0 ] [if personID != -1 [ask human personID [write who write harisma write onunitstime write "Levels123 and free time" write onL1time write onL2time write onL3time write freetime write ValuePerCareer show history]]]
  show  "Level 1 " ask units with [level = 1 ] [if personID != -1 [ask human personID [write who write harisma write onunitstime write "Levels123 and free time" write onL1time write onL2time write onL3time write freetime write ValuePerCareer show history]]]
  if one-of humans with [site != -1 and [level] of unit site = 1] != nobody  [write "Mean harisma" show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 1]) 2]
  show  "Level 2 " ask units with [level = 2 ] [if personID != -1 [ask human personID [write who write harisma write onunitstime write "Levels123 and free time" write onL1time write onL2time write onL3time write freetime write ValuePerCareer show history]]]
  if one-of humans with [site != -1 and [level] of unit site = 2] != nobody [write "Mean harisma" show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 2]) 2]
  show  "Level 3 " ask units with [level = 3 ] [if personID != -1 [ask human personID [write who write harisma write onunitstime write "Levels123 and free time" write onL1time write onL2time write onL3time write freetime write ValuePerCareer show history]]]
  if one-of humans with [site != -1 and [level] of unit site = 3] != nobody [write "Mean harisma" show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 3]) 2]
  show "Free humans. Who, Harisma, Time on units, Mean Value on Units, History with Units ID"
  ask humans with [site = -1][write who write Harisma write onunitstime write "Levels123 and free time" write onL1time write onL2time write onL3time write freetime write ValuePerCareer show history]
  if one-of humans with [site = -1] != nobody [write "Mean harisma" show precision (mean [harisma] of humans with [site = -1]) 2]

end
;; ------------------------------------------- ShowHarismaByLevels (Button) ------------------------------------------
to ShowHarismaByLevels
  ;; show to end of the simulation
  show "Mean Harisma by Levels" show "On units"
  show  "Level 0 " ask units with [level = 0 ] [if personID != -1 [show [harisma] of human personID ]]
  show  "Level 1 "
  if one-of humans with [site != -1 and [level] of unit site = 1] != nobody  [show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 1]) 2]
  show  "Level 2 "
  if one-of humans with [site != -1 and [level] of unit site = 2] != nobody [show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 2]) 2]
  show  "Level 3 "
  if one-of humans with [site != -1 and [level] of unit site = 3] != nobody [show precision (mean [harisma] of humans with [site != -1 and [level] of unit site = 3]) 2]
  show "Free humans."
  if one-of humans with [site = -1] != nobody [show precision (mean [harisma] of humans with [site = -1]) 2]

end
;; -------------------------------------------- MeanCalculation ------------------------------------------------------
to MeanCalculation
  ifelse count humans with [site != -1] > 0
  [set HarismaMean (sum [harisma] of humans with [site != -1]) / count humans with [site != -1] ]
  [set HarismaMean 0]

end
;; -------------------------------------------- Harisma Mean by Units Levels ---- DELETED !!!! ------------------------------------------------------

;; -------------------------------------------- OA   Index   Calculation -------------------------------------------------
to OAICalculation [Write2Log]
  ;; If there is no person in the subordinate position (level 3), OAIndex is 0
  if Write2Log [show "------------------OAIndexes Calculation---"]

  if Write2Log [show "---Level 3 ---"]
  ask units with [level = 3]
  [
    ifelse personID != -1 and HarismaMean != 0
    [set PQty 1 set HSum [harisma] of human personID set OAIndex precision ( HSum / (Difficulty)) 2  if Write2Log [write "OAIndex " show OAIndex]]
                                                                           ;;HSum / (HarismaMean * Difficulty)
    [set OAIndex 0]    ;; No person at unit, OAIndex = 0
  ]

  if Write2Log [show "---Level 2 ---"]
  ask units with [level = 2]
  [ let q who
    ifelse personID != -1 [set HSum [harisma] of human personID set PQty 1] [set HSum 0 set PQty 0 if Write2Log [write "No person at unit " show q]]

    let r count units with [level = 3 and my_boss = q and personID != -1]
    ifelse r > 0 and HarismaMean != 0
      [ let s sum [HSum] of units with [level = 3 and my_boss = q and personID != -1]
        let t sum [PQty] of units with [level = 3 and my_boss = q and personID != -1]
        set HSum HSum + s                  ;; s
        set PQty PQty + t

        set OAIndex precision ( HSum / (PQty * Difficulty)) 2  if Write2Log [write "OAIndex "  write OAIndex write " Human " show personID]
      ]
    [set OAIndex 0 if Write2Log [write "OAIndex " show OAIndex]]           ;; else for "ifelse r > 0 and HarismaMean != 0"
  ]

  if Write2Log [show "---Level 1 ---"]
  ask units with [level = 1]
  [ let q who
    ifelse personID != -1 [set HSum [harisma] of human personID set PQty 1] [set HSum 0 set PQty 0 if Write2Log [write "No person at unit " show q]]

    let r count units with [level = 2 and my_boss = q and personID != -1]
    ifelse r > 0 and HarismaMean != 0
      [ let s sum [HSum] of units with [level = 2 and my_boss = q and personID != -1]
        let t sum [PQty] of units with [level = 2 and my_boss = q and personID != -1]
        set HSum HSum + s
        set PQty PQty + t

        set OAIndex precision ( HSum / (PQty * Difficulty)) 2  if Write2Log [write "OAIndex "  write OAIndex write " Human " show personID]
      ]
    [set OAIndex 0 if Write2Log [write "OAIndex " show OAIndex]]           ;; else for "ifelse r > 0 and HarismaMean != 0"
  ]

  if Write2Log [show "---Level 0 ---"]
  ask units with [level = 0]
  [ let q who
    ifelse personID != -1 [set HSum [harisma] of human personID set PQty 1] [set HSum 0 set PQty 0 if Write2Log [write "No person at unit " show q]]

    let r count units with [level = 1 and my_boss = q and personID != -1]
    ifelse r > 0 and HarismaMean != 0
      [ let s sum [HSum] of units with [level = 1 and my_boss = q and personID != -1]
        let t sum [PQty] of units with [level = 1 and my_boss = q and personID != -1]
        set HSum HSum + s
        set PQty PQty + t

        set OAIndex precision ( HSum / (PQty * Difficulty)) 2  if Write2Log [write "OAIndex "  write OAIndex write " Human " show personID]
      ]
    [set OAIndex 0 if Write2Log [write "OAIndex " show OAIndex]]           ;; else for "ifelse r > 0 and HarismaMean != 0"
  ]
end
;; ------------------------------------------- ShowHumansList --------------------------------------------------
to ShowHumansList [UnitWho Print2Log]
 if Print2Log [show "----------------"]
 ifelse one-of units with [who = UnitWho] != nobody
 [
  let HList [] let Ulist []
  if [personID] of unit UnitWho = -1 [write "ShowHumansList. No Humans at unit " show UnitWho]   ;; Me, person, placed at unit UnitWho
  set HList lput [personID] of unit UnitWho HList set Ulist lput UnitWho UList

  ask units with [my_boss = UnitWho]
  [set HList lput personID HList set Ulist lput who UList]   ;; (humans are) suborders of person, placed at unit UnitWho

  if [my_boss] of unit UnitWho != -1      ;; only unit 0 has my_boss = -1
  [
    let f [my_boss] of unit UnitWho
 ;;--------------!!!!!  EXAMPLE of SINTAX ERROR !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 ;;--------------!!!!!  let f my_boss     NetLogo: You cannot use ShowHumansList in observer context  !!!!!!!!!!!!!!!!!
 ;;--------------!!!!!                    because ShowHumansList is tourtles only !!!!!!!!!!!!!!
    set HList lput ([personID] of unit f) HList set  Ulist lput f UList  ;; boss of person (or -1), placed at unit UnitWho

    ask units with [my_boss = f and who != UnitWho]                  ;; all units with the same boss, except me (Colleagues)
      [set HList lput (personID) HList set  Ulist lput who UList]    ;;
  ]
  if Print2Log [write "Humans " show HList write "Units" show Ulist]
  set HumanList HList
]
[write "ShowHumansList. It's no Unit with Who " show UnitWho] ;; ifelse one-of units with...
end

;;---------------------------------------------------------- SetCoalitionPerceptions ---------------------------------
to SetCoalitionPerceptions
ask Coalitions
  [
    set v who
    if AgressiveCoalition
          [ask humans with [coalitionID = v] [set PerceptOfOther  n-values HumansQty [-1] ] ]  ;; negative perception of other

    ask humans with [coalitionID = v]
    [ let lv who
      ask other humans with [coalitionID = v]
      [
        set PerceptOfOther replace-item lv PerceptOfOther 1 ;; it's my  (positive) perception of human v
      ]
    ]
  ]
end

;;---------------------------------------------------------- AssignPerceptions ----------------------------------------
to AssignPerceptions

  ;; ProbNegPercept is probability of Ngative percetion (-1) !!!!
  ask humans
  [
  let i 0
    while [i < HumansQty]
    [

    let prob random 100
    ifelse prob < ProbNegPercept [set PerceptOfOther lput -1 PerceptOfOther] [set PerceptOfOther lput 1 PerceptOfOther]
    set i i + 1
    ]
    set PerceptOfOther replace-item who PerceptOfOther 1 ;; it's my perception of myself
  ]

end

;;----------------------------------------------------------- ShowPerceptions (Button) ----------------------------------
to ShowPerceptions
  show "to ShowPerceptions"
  let i 0
  while [i < HumansQty]
  [ask human i  [write who show PerceptOfOther]
   set i i + 1
  ]
end

;;--------------------------------------------------  PowerCalc (Coalitions and No Coalition's memebers) -----------------
to TotalPowerCalc
 ;; coaliton's total power -----------------------------------------------------------------------------------------------
  set vp 0
  while [vp < CoalitionsQty0]
  [
  set v 0 set u 0
  ask humans [if (coalitionID = HumansQty + unitsqty + vp) and (site != -1) [set v v + ([value] of unit site )  set u u + [level] of unit site]    ]
  ask coalition (HumansQty + unitsqty + vp) [set power v set SumOfLevels u]
  set vp vp + 1
  ]
 ;; Total power of no Coalition's memebers -------------------------------------------------------------------------------
 set PowerNoCoalitionAgents 0
 ask humans with [(coalitionID = -1) and (site != -1)] [set PowerNoCoalitionAgents PowerNoCoalitionAgents + ([value] of unit site )]
 ;; Total power of no Coalition's memebers -------------------------------------------------------------------------------

end

;;------------------------------------------------------------------------------------------------------------------------
to DesignMyCoalition ;; Up to 6 memebers, for 5 members input one humans who twice

  set HumanList [0 1 2 3 4 5] let l length HumanList
  let i 0
  while [i < l]
  [
   set HumanList replace-item i HumanList read-from-string user-input i ;; INPUT ERROR CONTROL MUST BE DO !!!!
                                                                        ;; Coalition members are from HumansList (global var)

    ;; carefully [ commands1 ] [ print error-message stop ]
   set i i + 1
  ]

  if length HumanList > 0 and CoalitionsQty0 < 5 and one-of humans with [member? who HumanList] != nobody
  [
     let color_list [red blue black green yellow]
     set CoalitionsQty0 CoalitionsQty0 + 1
     set CoalitionsQty CoalitionsQty0
     create-ordered-coalitions 1 [set xcor max-pxcor set ycor min-pycor + random 10 set color red  set shape "flag" set v who set Objectives_list [] ]
     ask humans [if (member? who HumanList) [set coalitionID v set color item (CoalitionsQty0 - 1) color_list]]
  ]

SetCoalitionPerceptions ;; Reassign positive perceptions for coalition's members
end
;; ------------------------------------------------------------------------------------------------------------------------
to Results ;; 4 experimental data collection
  set v (sum [harisma] of humans  with [site != -1])
  set u (sum [harisma] of humans  with [site  = -1]) ;; / PotentialProductivity * 100 ;; %%
  set ActualProductivityHistory lput v ActualProductivityHistory
  set MissedProductivityHistory lput u MissedProductivityHistory

  ifelse (count humans  with [site  != -1] > 0) [set v  mean [harisma] of humans  with [site  != -1]] [set v 0]
  ifelse (count humans  with [site  = -1] > 0) [set u  mean [harisma] of humans  with [site  = -1]] [set u 0]
  set MeanActualProductivityHistory lput v MeanActualProductivityHistory
  set MeanMissedProductivityHistory lput u MeanMissedProductivityHistory

end
@#$#@#$#@
GRAPHICS-WINDOW
278
10
887
620
-1
-1
7.241
1
12
1
1
1
0
1
1
1
-41
41
-41
41
1
1
1
ticks
20.0

BUTTON
27
297
95
330
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
177
297
245
330
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
95
297
176
330
go once
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
25
27
132
60
HumansQty
HumansQty
1
300
147.0
1
1
NIL
HORIZONTAL

SLIDER
134
27
242
60
turt_size
turt_size
1
5
4.0
1
1
NIL
HORIZONTAL

CHOOSER
913
407
1017
452
subset
subset
"OriginalColor" "Heads" "Departments" "Coal Objectives" "Unit"
2

INPUTBOX
1017
407
1077
467
AgentWho
430.0
1
0
Number

BUTTON
914
454
997
487
ShowSubset
showSubset
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
93
10
221
28
SETUP BUTTONS
14
0.0
1

TEXTBOX
45
279
264
298
OTHER BUTTONS HAVE EFFECT ON FLY
11
0.0
1

TEXTBOX
96
368
223
386
Parameters for Decisions
11
0.0
1

MONITOR
197
546
255
591
UnitsQty
UnitsQty
0
1
11

SLIDER
20
506
146
539
InfluenceRadius
InfluenceRadius
0
50
20.0
1
1
NIL
HORIZONTAL

SLIDER
1
62
132
95
FirstDistrDensity
FirstDistrDensity
0
1
0.7
.1
1
NIL
HORIZONTAL

MONITOR
31
546
121
591
NIL
LastChangeTick
17
1
11

MONITOR
121
546
196
591
NIL
ChangesQty
17
1
11

SLIDER
22
97
131
130
CoalitionsQty
CoalitionsQty
0
5
0.0
1
1
NIL
HORIZONTAL

SLIDER
2
166
94
199
Lev1Units
Lev1Units
1
10
4.0
1
1
NIL
HORIZONTAL

SLIDER
95
166
187
199
Lev2Units
Lev2Units
1
10
5.0
1
1
NIL
HORIZONTAL

SLIDER
186
166
278
199
Lev3Units
Lev3Units
1
10
5.0
1
1
NIL
HORIZONTAL

SLIDER
134
98
263
131
NoCoalition%
NoCoalition%
0
100
78.0
1
1
NIL
HORIZONTAL

SLIDER
2
202
94
235
L0to1Dist
L0to1Dist
3
20
7.0
1
1
NIL
HORIZONTAL

SLIDER
92
202
184
235
L1to2Dist
L1to2Dist
3
20
6.0
1
1
NIL
HORIZONTAL

SLIDER
184
202
277
235
L2to3Dist
L2to3Dist
3
25
12.0
1
1
NIL
HORIZONTAL

MONITOR
963
10
1044
55
2. FH2EmUn
PB1
17
1
11

MONITOR
891
10
961
55
1. VolDism
VDm
17
1
11

MONITOR
890
56
969
101
4. DmWorst
DmWorst
17
1
11

MONITOR
65
594
143
639
Free Units
Count units with [personID = -1]
17
1
11

MONITOR
971
355
1028
400
Level1
count units with [personID = -1 and Level = 1]
17
1
11

MONITOR
1030
355
1086
400
Level2
count units with [personID = -1 and Level = 2]
17
1
11

MONITOR
1085
355
1142
400
Level3
count units with [personID = -1 and Level = 3]
17
1
11

MONITOR
914
355
971
400
Level0
count units with [personID = -1 and Level = 0]
17
1
11

BUTTON
905
504
1107
537
Show Personal Objectives
ask humans [write coalitionID show MyObjectives_list]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
1079
539
1213
572
PersonsDoublingUnits
PersonsDoublingUnits
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
1079
576
1214
609
UnitsDoublingPersons
UnitsDoublingPersons
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
952
340
1013
358
Free Units
13
0.0
1

MONITOR
145
594
227
639
FreePersons
count humans with [site = -1]
17
1
11

MONITOR
1027
144
1094
189
9. P2Frnd
P2Frnd
17
1
11

PLOT
1204
599
1456
752
PowerPerHuman
NIL
NIL
0.0
10.0
0.0
5.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" "plot [power] of coalition (HumansQty + unitsqty) / count humans with [coalitionID = HumansQty + unitsqty]"
"pen-1" 1.0 2 -7500403 true "" "plot PowerNoCoalitionAgents / count humans with [coalitionID = -1]"
"pen-2" 1.0 0 -13345367 true "" "plot [power] of coalition (HumansQty + unitsqty + 1) / count humans with [coalitionID = HumansQty + unitsqty + 1]"
"pen-3" 1.0 0 -13628663 true "" "plot [power] of coalition (HumansQty + unitsqty + 2) / count humans with [coalitionID = HumansQty + unitsqty + 2]"
"pen-4" 1.0 0 -15040220 true "" "plot [power] of coalition (HumansQty + unitsqty + 3) / count humans with [coalitionID = HumansQty + unitsqty + 3]"
"pen-5" 1.0 0 -1184463 true "" "plot [power] of coalition (HumansQty + unitsqty + 4) / count humans with [coalitionID = HumansQty + unitsqty + 4]"

SLIDER
45
333
137
366
LastTick
LastTick
200
1000
200.0
100
1
NIL
HORIZONTAL

MONITOR
1043
10
1124
55
3. BH2EmUn
PB2
17
1
11

MONITOR
1142
188
1199
233
DmAll
DmAll
17
1
11

MONITOR
1072
189
1135
234
13. UpGr
UpGr
17
1
11

SLIDER
139
333
235
366
Difficulty
Difficulty
100
150
150.0
50
1
NIL
HORIZONTAL

BUTTON
890
304
992
337
HumansHistory
ShowHumansHistory
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
889
234
1077
267
ShowHumansList of AgentWho
ShowHumansList AgentWho true
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
906
540
1050
573
ShowPerceptions
ShowPerceptions
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
134
62
263
95
ProbNegPercept
ProbNegPercept
0
100
70.0
10
1
NIL
HORIZONTAL

BUTTON
889
270
974
303
UnitsHistory
ShowUnitsHistory
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
1473
10
1664
750
OAIndex by Levels
NIL
NIL
0.0
10.0
0.0
1.2
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "if (ticks > 10) [plot mean [OAIndex] of units with [level = 1]]"
"pen-1" 1.0 0 -13345367 true "" "if (ticks > 10) [plot mean [OAIndex] of units with [level = 2]]"
"pen-2" 1.0 0 -2674135 true "" "if (ticks > 10) [plot mean [OAIndex] of units with [level = 3]]"
"pen-3" 1.0 0 -7500403 true "plot 0.5" "plot 0.5"
"pen-4" 1.0 0 -955883 true "" "plot 1"

BUTTON
977
269
1112
302
DecisionsbyLevels
DecisionsbyLevels
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
1077
234
1184
267
ShowSuborders
ShowSuborders
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
1116
269
1193
302
OA Indexes
OAICalculation true
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
1094
144
1181
189
10. P2FrndL3
P2Frnd3
17
1
11

MONITOR
963
144
1028
189
8. P2NBU
P2NBU
17
1
11

MONITOR
890
189
975
234
11. DwnGrDism
DwnGrDism
17
1
11

MONITOR
974
189
1072
234
12. DwnGrWrst
DwnGr
17
1
11

BUTTON
1109
504
1257
537
ShowCoalitionsResults
ShowCoalitionsResults
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
1209
10
1469
195
Total Productivity
NIL
NIL
0.0
0.0
0.0
0.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot sum [harisma] of humans  with [site != -1]\n\n"
"pen-1" 1.0 2 -7500403 true "" "plot sum [harisma] of humans  with [site = -1]"
"pen-2" 1.0 0 -2674135 true "" "plot sum [harisma] of max-n-of (min (list UnitsQty HumansQty)) humans [harisma] "

MONITOR
890
101
1042
146
6.1 GetGoodSub Busy Person
GetSub
17
1
11

MONITOR
1042
101
1193
146
6.2 GetGoodSub Free Person
GetFreSub
17
1
11

SWITCH
148
506
246
539
LogWrite
LogWrite
1
1
-1000

SWITCH
83
236
205
269
RegularPlace
RegularPlace
0
1
-1000

PLOT
1277
449
1455
597
FreeUnits
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot count units with [PersonID = -1]"

BUTTON
993
303
1096
336
HarismaByLevels
ShowHarismaByLevels
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
152
422
263
455
PerceptON
PerceptON
0
1
-1000

SLIDER
193
386
285
419
GradeH
GradeH
0
1
0.8
.05
1
NIL
HORIZONTAL

SLIDER
101
386
193
419
GradeM
GradeM
0
GradeH
0.7
.05
1
NIL
HORIZONTAL

SLIDER
8
386
100
419
GradeL
GradeL
0
GradeM
0.6
.05
1
NIL
HORIZONTAL

PLOT
1209
197
1468
391
Productivity Per Human
NIL
NIL
0.0
10.0
80.0
10.0
true
false
"" ""
PENS
"pen-3" 1.0 0 -16514813 true "" "plot (sum [harisma] of humans  with [site != -1])/(count humans  with [site != -1])"
"pen-4" 1.0 2 -7500403 true "" "plot (sum [harisma] of humans  with [site = -1])/(count humans  with [site = -1])"
"pen-2" 1.0 0 -7500403 true "" "plot mean [harisma] of humans"

SWITCH
8
470
147
503
OneTimeAsgn
OneTimeAsgn
0
1
-1000

MONITOR
970
56
1049
101
5. GetAnySub
GetAnySub
17
1
11

SWITCH
148
470
246
503
Initiative
Initiative
0
1
-1000

MONITOR
890
145
963
190
7. ReplSub
ReplSub
17
1
11

CHOOSER
8
422
149
467
ScreenPlay
ScreenPlay
"SP1 (No Lev2 Ctrl)" "SP2 (3 Points Ctrl)" "SP3 (No Lev 0 Ctrl)" "SP4 (Random)" "SP5 No Control"
1

PLOT
905
582
1065
702
Productivity on Units
NIL
NIL
0.0
1000.0
0.0
50.0
true
false
"set-plot-x-range 0 200\nset-histogram-num-bars 20\n" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [harisma] of humans with [site != -1]"

SWITCH
64
131
219
164
AgressiveCoalition
AgressiveCoalition
0
1
-1000

@#$#@#$#@
## WHAT IS IT?

This is a model of organizational behavior in the hierarchy in which personnel decisions are made.


## HOW IT WORKS
The World of the model is a four-level hierarchical structure, consisting of shuff positions of the top manager (zero level of the hierarchy), first-level managers who are subordinate to the top manager, second-level managers (subordinate to the first-level managers) and positions of employees (the third level of the hierarchy). ) subordinated to the second-level managers. Such a hierarchy is a tree, i.e. each position, with the exception of the position of top manager, has a single boss.

Agents in the model are persons occupying the specified positions, the number of persons is set by the slider (HumansQty). Personas have some operational performance (harisma, an unfortunate attribute name left over from the first edition of the model)) and a sense of other personas' own perceptions. Performance values are distributed over the ensemble of persons according to the normal law with some mean value and variance.

The value of perception by agents of each other is positive or negative (implemented in the model as numerical values equal to +1 and -1). The distribution of perceptions over an ensemble of persons is implemented as a random variable specified by the probability of negative perception, the value of which is set by the control elements of the model interface. The numerical value of the probability equal to 0 corresponds to the case in which all persons positively perceive each other (the numerical value of the random variable is equal to 1, which corresponds to the positive perception of the other person by the individual).

The hierarchy is occupied with operational activity, the degree of intensity of which is set by the external parameter Difficulty. The level of productivity of each manager OAIndex is equal to the level of productivity of the department he leads and is the ratio of the sum of productivity of employees subordinate to the head to the level of complexity of the work Difficulty. An increase in the numerical value of Difficulty leads to a decrease in the OAIndex for all subdivisions of the hierarchy. The managerial meaning of the OAIndex indicator is the percentage of completion of the load specified for the hierarchy as a whole, i.e. the ratio of the actual performance of the structural subdivisions of the hierarchy to the required performance, the level of which is specified by the value of the Difficulty parameter.

A specific hierarchy, the behavior of which is investigated in the course of a simulation experiment, is characterized by the following parameters:
• number of persons and positions (hierarchy structure is set by sliders (Lev1Units, Lev1Units etc.));
• the value of positions (increases linearly with a decrease in the level from the third to zero), which is the amount of remuneration for the work of a person in this position, and thus the value of a particular position for a person;
• percentage of positions occupied by persons at the initial moment of the experiment (FirstDistrDensity);
• probability of negative perception of other persons by persons (ProbNegPercept);
• individual performance of persons;
• prohibition or absence of a prohibition on repeated (after release) occupation of positions by persons.

Decisions are:

1. D_VoluntaryDismisial (mutual perception influences the decision)
2. Decision_EU_IB_FH (mutual perception influences the decision)
3. Decision_EU_IB_BH (mutual perception influences the decision)
4. D_DismissWorst
5. D_GetAnySuborder4L3
6. D_GetGoodSuborder4L3 (mutual perception influences the decision)
7. D_ReplaceSuborder4L3onFreeHuman (mutual perception influences the decision)
8. D_Pass2NoBossUnit
9. D_Pass2FrendLev3 (mutual perception influences the decision)
10. D_Pass2FrendSameLevel (mutual perception influences the decision)
11. D_DownGradeOrDismiss (mutual perception influences the decision)
12. D_DownGradeWorstBoss
13. D_UpGradeBestBoss

The idea of the model is that the hierarchy does some operational work and is described by such characteristics as structure (number and interrelation of positions) and material, filling these positions (persons with their individual performance). A particular hierarchy is under certain external pressure (performance level requirement) and is characterized by the internal state of the material (the distribution of the perceptions of others over the ensemble of persons). Let some types of personnel decisions be allowed and the rules for their adoption are indicated (availability of solutions for positions of one level or another, performance criteria, local distribution of perceptions). Let's call a control scenario a set of personnel decisions together with the regulations for their adoption.

Key Scenarios (set by change ScreenPlay):
• "Inaction" ("SP5 No Control"). Only individual solutions are allowed (## 1-3). Managers do not make decisions about subordinates.
• "Chaos" ("SP4 (Random)"). Individual decisions ## 1-3 are allowed, in addition, managers randomly, with equal probability, make one of the decisions ## 4-13.
• "Dictatorship" ("SP2 (3 Points Ctrl)"). Individual decisions ## 1-3 are allowed, in addition, managers make one of the decisions ## 4-13, depending on whether the current value of the OAIndex performance index falls into one or another range of values specified by the GradeL < GradeM < GradeH parameters. Thus, depending on the current value of OAIndex, managers vacate positions from less productive performers and replace them with more productive ones, guided, among other things, by the personal perception of subordinates. The intervals of performance values are set by the GradeL, GradeM,GradeH sliders.

A series of simulation experiments (see BehaviorSpace) is designed to give an answer about the influence of the control scenario on the performance of certain types of hierarchy under various initial conditions.

You can create a coalition - a group of people in which mutual perceptions are positive.
The number of coalitions is set by the CoalitionsQty slider. If you set (at SETUP) the switch AgressiveCoalition ON, coalitions will become "aggressive", in the sense that their members will negatively perceive everyone who is not a member of this coalition.

The model allows you to collect statistics on the number of decisions made, on the performance of individual groups of agents, track the career path of any agent (positions occupied by him during the experiment and the time spent in positions), the number and types of decisions made at different levels of the hierarchy, etc.

## HOW TO USE IT

The SETUP button forms hierarchy and set perceptions (ProbNegPercept slider), control scenario selection (ScreenPlay), three-point control parameters (GradeL, GradeM, GradeH), and FirstDistrDensity.

The GO button keeps the model running until ticks less value of LastTick slider.

Reports: 
• Total Productivity (for humans un units versus theoretical productivity (for the hierarchy) max, plot)
• Productivity Per Human (mean for humans un units and free humans versus total mean productivity, plot)
• FreeUnits (plot)
• OAIndex by Levels (plot)
• PowerPerHuman (mean Value of positions occupied across coalitions), plot)
• Harisma (personal productivity distribution, plot)

• Decisions 1-13 quantity (screens)
• Free units by levels (screens)
• LastChangeTick, Total ChangesQty, Units quantity (screens)
• Total free units and persons (screens)

• Subsets of units (buttons Departments, Heads, coalition's objectives, Unit(by who)) (in the World window)

• Data about some result  of run, collected by such procedures (buttons) as DecisionsbyLevels, ShowHumansHistory, ShowSuborders, Show Personal Objectives and so on (numbers in Observer window)


## THINGS TO NOTICE

1) in a comfortable environment (moderate operational load, mostly positive interpersonal perceptions), the formal management scenario (“Dictatorship”) gives the best result in terms of productivity of the hierarchy as a whole. The inaction of managers (approving incoming applications for positions according to the criterion of a positive personal perception of the candidate) and the random choice of a personnel decision (“Chaos”) according to this criterion differ slightly.
2) in an aggressive environment (high operational load, mostly negative interpersonal perceptions)
   •  inaction of leaders does not contribute to the growth of productivity of the              hierarchy as a whole;
   •  a random selection of a personnel decision from a predetermined list gives better         results than inaction;
   •  the scenario of control on a formal basis ("Dictatorship") gives the best result in       terms of productivity of the hierarchy as a whole.

## THINGS TO TRY

Try any combination of number of persons and units. 
Try hierarchy structures like 1-1-1-10 or 1-10-1-1. 
Try any combination ProbNegPercept and Difficulty. 
Try to create several coalitions. 
Try switch PerceptON, OneTimeAsgn and Initiative sliders.
Try run experiments (see BehaviorSpace). XLS-parser for extracting data from table output file is attached.


## EXTENDING THE MODEL

1. New control scenarios can be built as a combination of certain decisions.
2. New types of decisions can be added.
3. Decisions that take into account the membership of the decision maker or the object of the decision in the coalition are not implemented in this edition of the model.

## NETLOGO FEATURES

The world topology is a "torus".
Lists are used to remember career history and store time series.
Service variables are used to transfer values from one context enclosed in brackets [] to another.
The model code itself does not contain any fragments of an innovative or particularly instructive nature, it is quite trivial.

## CREDITS AND REFERENCES

The model represents the first approximation to the implementation of the concept of mutual perception in the hierarchy formulated by the author in the works:
1.	Ivan Smarzhevskiy. The concept of mutual perception of persons in a hierarchy.  Journal of Economy and entrepreneurship, 2018, Vol. 12, Nom. 12, p. 1120-1124. (И.А. Смаржевский, Концепция взаимного восприятия персон, занимающих позиции в иерархической структуре. // Экономика и предпринимательство. 2018. № 12. С.1120-1124).
2.	Ivan Smarzhevskiy. The development of the concept of mutual perception of persons in a hierarchy: the dynamics of perceptions.  Journal of Economy and entrepreneurship, 2019, Vol. 13, Nom. 1, p. 830-833 (И.А. Смаржевский, Развитие концепции взаимного восприятия персон в иерархической структуре: динамика восприятий // Экономика и предпринимательство. 2019. № 1. С. 830-833.)


## HOW TO CITE

If you mention this model or the NetLogo software in a publication, we ask that you include the citations below.

For the model itself:


* Ivan Smarzhevskiy (2022). Personnel decisions in the hierarchy.
https://www.comses.net/codebase-release/b4ca1801-b048-4568-93dc-70d56087cc5f/

Please cite the NetLogo software as:

* Wilensky, U. (1999). NetLogo. http://ccl.northwestern.edu/netlogo/. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

## COPYRIGHT AND LICENSE

Copyright Ivan Smarzhevskiy 2022 

<!-- 2022 -->
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.2.2
@#$#@#$#@
setup
repeat 20 [ go ]
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="H152-U76,FD 0.7,200 ticks Mean Indicators" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>precision PotentialProductivity 2</metric>
    <metric>precision mean ActualProductivityHistory 2</metric>
    <metric>precision mean [harisma] of humans 2</metric>
    <metric>precision mean MeanActualProductivityHistory 2</metric>
    <metric>VDm</metric>
    <metric>count units with [personID = -1 and Level = 0]</metric>
    <metric>count units with [personID = -1 and Level = 1]</metric>
    <metric>count units with [personID = -1 and Level = 2]</metric>
    <metric>count units with [personID = -1 and Level = 3]</metric>
    <enumeratedValueSet variable="ScreenPlay">
      <value value="&quot;SP2 (3 Points Ctrl)&quot;"/>
      <value value="&quot;SP4 (Random)&quot;"/>
      <value value="&quot;SP5 No Control&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Difficulty">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ProbNegPercept">
      <value value="30"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="H290-U145, FD 0.7,200 ticks Mean Indicators" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>precision PotentialProductivity 2</metric>
    <metric>precision mean ActualProductivityHistory 2</metric>
    <metric>precision mean [harisma] of humans 2</metric>
    <metric>precision mean MeanActualProductivityHistory 2</metric>
    <metric>VDm</metric>
    <metric>count units with [personID = -1 and Level = 0]</metric>
    <metric>count units with [personID = -1 and Level = 1]</metric>
    <metric>count units with [personID = -1 and Level = 2]</metric>
    <metric>count units with [personID = -1 and Level = 3]</metric>
    <enumeratedValueSet variable="ScreenPlay">
      <value value="&quot;SP2 (3 Points Ctrl)&quot;"/>
      <value value="&quot;SP4 (Random)&quot;"/>
      <value value="&quot;SP5 No Control&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Difficulty">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ProbNegPercept">
      <value value="30"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="H290-U145, FD 0.7,200 ticks 150 70" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>precision PotentialProductivity 2</metric>
    <metric>precision mean ActualProductivityHistory 2</metric>
    <metric>precision mean [harisma] of humans 2</metric>
    <metric>precision mean MeanActualProductivityHistory 2</metric>
    <metric>VDm</metric>
    <metric>count units with [personID = -1 and Level = 0]</metric>
    <metric>count units with [personID = -1 and Level = 1]</metric>
    <metric>count units with [personID = -1 and Level = 2]</metric>
    <metric>count units with [personID = -1 and Level = 3]</metric>
    <enumeratedValueSet variable="ScreenPlay">
      <value value="&quot;SP2 (3 Points Ctrl)&quot;"/>
      <value value="&quot;SP4 (Random)&quot;"/>
      <value value="&quot;SP5 No Control&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Difficulty">
      <value value="150"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ProbNegPercept">
      <value value="70"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="H145-U145, FD 0.7,200 ticks 150 70" repetitions="100" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>precision PotentialProductivity 2</metric>
    <metric>precision mean ActualProductivityHistory 2</metric>
    <metric>precision mean [harisma] of humans 2</metric>
    <metric>precision mean MeanActualProductivityHistory 2</metric>
    <metric>VDm</metric>
    <metric>count units with [personID = -1 and Level = 0]</metric>
    <metric>count units with [personID = -1 and Level = 1]</metric>
    <metric>count units with [personID = -1 and Level = 2]</metric>
    <metric>count units with [personID = -1 and Level = 3]</metric>
    <enumeratedValueSet variable="ScreenPlay">
      <value value="&quot;SP2 (3 Points Ctrl)&quot;"/>
      <value value="&quot;SP4 (Random)&quot;"/>
      <value value="&quot;SP5 No Control&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Difficulty">
      <value value="150"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ProbNegPercept">
      <value value="70"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
